<?php
// vim: set ts=4 sw=4 sts=4 et:

/**
 * X-Cart
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the software license agreement
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.x-cart.com/license-agreement.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to licensing@x-cart.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not modify this file if you wish to upgrade X-Cart to newer versions
 * in the future. If you wish to customize X-Cart for your needs please
 * refer to http://www.x-cart.com/ for more information.
 *
 * @category  X-Cart 5
 * @author    Qualiteam software Ltd <info@x-cart.com>
 * @copyright Copyright (c) 2011-2015 Qualiteam software Ltd <info@x-cart.com>. All rights reserved
 * @license   http://www.x-cart.com/license-agreement.html X-Cart 5 License Agreement
 * @link      http://www.x-cart.com/
 */

namespace XLite\Module\XC\Reviews\Model\Repo;

/**
 * Reviews repository
 *
 */
class Review extends \XLite\Model\Repo\ARepo
{

    // {{{ Search

    const SEARCH_TYPE_REVIEWS_ONLY = 100;
    const SEARCH_TYPE_RATINGS_ONLY = 200;

    const SEARCH_ADDITION_DATE = 'additionDate';
    const SEARCH_STATUS = 'status';
    const SEARCH_REVIEWER_NAME = 'reviewerName';
    const SEARCH_EMAIL = 'email';
    const SEARCH_REVIEW = 'review';
    const SEARCH_PRODUCT = 'product';
    const SEARCH_KEYWORDS = 'keywords';
    const SEARCH_RATING = 'rating';
    const SEARCH_LIMIT = 'limit';
    const SEARCH_ORDERBY = 'orderBy';
    const SEARCH_ZONE = 'zone';
    const SEARCH_IP = 'ip';
    const SEARCH_DATE_RANGE = 'dateRange';
    const SEARCH_TYPE = 'type';

    const SEARCH_ZONE_CUSTOMER = 'customer';

    /**
     * Common search
     *
     * @param \XLite\Core\CommonCell $cnd       Search condition
     * @param boolean                $countOnly Return items list or only its size OPTIONAL
     *
     * @return \Doctrine\ORM\PersistentCollection|integer
     */
    public function search(\XLite\Core\CommonCell $cnd, $countOnly = false)
    {
        $queryBuilder = $this->createQueryBuilder('r');
        $this->currentSearchCnd = $cnd;

        foreach ($this->currentSearchCnd as $key => $value) {
            $this->callSearchConditionHandler($value, $key, $queryBuilder, $countOnly);
        }

        return $countOnly
            ? $this->searchCount($queryBuilder)
            : $this->searchResult($queryBuilder);
    }

    /**
     * Search count only routine.
     *
     * @param \Doctrine\ORM\QueryBuilder $qb Query builder routine
     *
     * @return \Doctrine\ORM\PersistentCollection|integer
     */
    public function searchCount(\Doctrine\ORM\QueryBuilder $qb)
    {
        $qb->select('COUNT(DISTINCT r.id)');

        return intval($qb->getSingleScalarResult());
    }

    /**
     * Search result routine.
     *
     * @param \Doctrine\ORM\QueryBuilder $qb Query builder routine
     *
     * @return \Doctrine\ORM\PersistentCollection|integer
     */
    public function searchResult(\Doctrine\ORM\QueryBuilder $qb)
    {
        return $qb->getResult();
    }

    /**
     * Search sum of rating
     *
     * @param \XLite\Core\CommonCell $cnd Search condition
     *
     * @return \Doctrine\ORM\PersistentCollection|integer
     */
    public function searchTotalRating(\XLite\Core\CommonCell $cnd)
    {
        $queryBuilder = $this->createQueryBuilder('r');
        $this->currentSearchCnd = $cnd;

        $countOnly = false;

        foreach ($this->currentSearchCnd as $key => $value) {
            $this->callSearchConditionHandler($value, $key, $queryBuilder, $countOnly);
        }

        $queryBuilder->select('SUM(r.rating)');

        return intval($queryBuilder->getSingleScalarResult());
    }

    /**
     * Call corresponded method to handle a search condition
     *
     * @param mixed                      $value        Condition data
     * @param string                     $key          Condition name
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param boolean                    $countOnly    Count only flag
     *
     * @return void
     */
    protected function callSearchConditionHandler($value, $key, \Doctrine\ORM\QueryBuilder $queryBuilder, $countOnly)
    {
        if ($this->isSearchParamHasHandler($key)) {
            $this->{'prepareCnd' . ucfirst($key)}($queryBuilder, $value, $countOnly);
        }
    }

    /**
     * Check if param can be used for search
     *
     * @param string $param Name of param to check
     *
     * @return boolean
     */
    protected function isSearchParamHasHandler($param)
    {
        return in_array($param, $this->getHandlingSearchParams());
    }

    /**
     * Return list of handling search params
     *
     * @return array
     */
    protected function getHandlingSearchParams()
    {
        return array(
            static::SEARCH_ADDITION_DATE,
            static::SEARCH_DATE_RANGE,
            static::SEARCH_STATUS,
            static::SEARCH_PRODUCT,
            static::SEARCH_KEYWORDS,
            static::SEARCH_RATING,
            static::SEARCH_LIMIT,
            static::SEARCH_ORDERBY,
            static::SEARCH_ZONE,
            static::SEARCH_IP,
            static::SEARCH_TYPE,
        );
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndAdditionDate(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {

        if (is_array($value)) {
            $value = array_values($value);
            $start = empty($value[0]) ? null : intval($value[0]);
            $end = empty($value[1]) ? null : intval($value[1]);

            if ($start == $end) {
                return;
            }

            if ($start) {
                $queryBuilder->andWhere('r.additionDate >= :start')
                    ->setParameter('start', $start);
            }

            if ($end) {
                $queryBuilder->andWhere('r.additionDate <= :end')
                    ->setParameter('end', $end);
            }
        }

    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param integer                    $value        Condition data
     *
     * @return void
     */
    protected function prepareCndDateRange(\Doctrine\ORM\QueryBuilder $queryBuilder, $value)
    {
        if (!empty($value)) {
            list($start, $end) = \XLite\View\FormField\Input\Text\DateRange::convertToArray($value);
            if ($start) {
                $queryBuilder->andWhere('r.additionDate >= :start')
                    ->setParameter('start', $start);
            }

            if ($end) {
                $queryBuilder->andWhere('r.additionDate <= :end')
                    ->setParameter('end', $end);
            }
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndType(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        if (\XLite\Module\XC\Reviews\Model\Repo\Review::SEARCH_TYPE_REVIEWS_ONLY == $value) {
            $queryBuilder->andWhere('r.review != :type')
                ->setParameter('type', '');

        } elseif (\XLite\Module\XC\Reviews\Model\Repo\Review::SEARCH_TYPE_RATINGS_ONLY == $value) {
            $queryBuilder->andWhere('r.review = :type')
                ->setParameter('type', '');
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndZone(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        $values = array_values($value);

        if (isset($values[0])
            && $values[0] == self::SEARCH_ZONE_CUSTOMER
            && \XLite\Core\Config::getInstance()->XC->Reviews->disablePendingReviews == true
        ) {
            $statusCondition = 'r.status = ' . \XLite\Module\XC\Reviews\Model\Review::STATUS_APPROVED;

            if ($values[1] instanceof \XLite\Model\Profile) {
                $queryBuilder->linkLeft('r.profile', 'u');

                $queryBuilder->andWhere($statusCondition . ' OR u.profile_id = :profileId')
                    ->setParameter('profileId', $values[1]->getProfileId());

            } else {
                $reviewIds = \XLite\Core\Session::getInstance()->reviewIds;
                if (is_array($reviewIds) && !empty($reviewIds)) {
                    $queryBuilder->andWhere($statusCondition . ' OR r.id IN (' . implode(', ', $reviewIds) . ')');
                } else {
                    $queryBuilder->andWhere($statusCondition);
                }
            }
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndStatus(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        if (isset($value) && '%' != $value) {
            $queryBuilder->andWhere('r.status = :status')
                ->setParameter('status', $value);
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndRating(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        if (isset($value) && '%' != $value) {
            if (!is_array($value)) {
                $value = array($value);
            }

            $value = array_filter($value);

            if (1 == count($value)) {
                $queryBuilder->andWhere('r.rating = :rating')
                    ->setParameter('rating', $value[0]);

            } elseif (1 < count($value)) {
                $queryBuilder->andWhere($queryBuilder->expr()->in('r.rating', $value));
            }
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndReviewerName(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        if ($value) {
            $queryBuilder->andWhere('r.reviewerName LIKE :reviewerName OR r.email LIKE :reviewerName')
                ->setParameter('reviewerName', '%' . $value . '%');
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndIp(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        if ($value) {
            $queryBuilder->andWhere('r.ip = :ip')
                ->setParameter('ip', ip2long($value));
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndKeywords(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        if ($value) {
            $queryBuilder->linkInner('r.product', 'p')
                ->innerJoin('p.translations', 'translations');

            $conditions = array(
                'r.reviewerName LIKE :keywords',
                'r.email LIKE :keywords',
                'p.sku LIKE :keywords',
                'translations.name LIKE :keywords'
            );

            $queryBuilder->andWhere(implode(' OR ', $conditions))
                ->setParameter('keywords', '%' . $value . '%');
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndEmail(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        if ($value) {
            $queryBuilder->andWhere('r.email LIKE :email')
                ->setParameter('email', '%' . $value . '%');
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndReview(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        if ($value) {
            $queryBuilder->andWhere('r.review LIKE :review')
                ->setParameter('review', '%' . $value . '%');
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param mixed                      $value        Condition data
     *
     * @return void
     */
    protected function prepareCndProduct(\Doctrine\ORM\QueryBuilder $queryBuilder, $value)
    {
        if ($value instanceof \XLite\Model\Product) {
            $queryBuilder->linkInner('r.product', 'p');

            $queryBuilder->andWhere('p.product_id = :productId')
                ->setParameter('productId', $value->getProductId());
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param mixed                      $value        Condition data
     *
     * @return void
     */
    protected function prepareCndProfile(\Doctrine\ORM\QueryBuilder $queryBuilder, $value)
    {
        if ($value instanceof \XLite\Model\Profile) {
            $queryBuilder->linkInner('r.profile', 'u');

            $queryBuilder->andWhere('u.profile_id = :profileId')
                ->setParameter('profileId', $value->getProfileId());
        }
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array                      $value        Condition data
     *
     * @return void
     */
    protected function prepareCndLimit(\Doctrine\ORM\QueryBuilder $queryBuilder, array $value)
    {
        $queryBuilder->setFrameResults($value);
    }

    /**
     * Prepare certain search condition
     *
     * @param \Doctrine\ORM\QueryBuilder $queryBuilder Query builder to prepare
     * @param array|string               $value        Condition data
     * @param boolean                    $countOnly    "Count only" flag. Do not need to add "order by" clauses
     *                                                 if only count is needed.
     *
     * @return void
     */
    protected function prepareCndOrderBy(\Doctrine\ORM\QueryBuilder $queryBuilder, $value, $countOnly)
    {
        if (!$countOnly) {
            list($sort, $order) = $this->getSortOrderValue($value);

            if ($sort) {
                $queryBuilder->addOrderBy($sort, $order);
            }
        }
    }

    // {{{ Meta

    public function setUseForMeta($productId, $reviewId)
    {
        $review = $this->findOneBy(array(
            'id' => $reviewId,
            'product' => $productId,
        ));

        if ($review) {
            $this->getQueryBuilder()
                ->update($this->_entityName, 'r')
                ->set('r.useForMeta', 0)
                ->where('r.product = :product')
                ->setParameter('product', $productId)
                ->execute();

            $review->setUseForMeta(true);
            $review->update();
        }
    }

    /**
     * Returns review for meta description
     *
     * @param \XLite\Model\Product $product Product
     *
     * @return \XLite\Module\XC\Reviews\Model\Review
     */
    public function findOneForMeta($product)
    {
        $queryBuilder = $this->defineFindOneForMeta($product);

        return $queryBuilder->getSingleResult();
    }

    /**
     * Returns query builder
     *
     * @param \XLite\Model\Product $product Product
     *
     * @return \Doctrine\ORM\QueryBuilder
     */
    protected function defineFindOneForMeta($product)
    {
        $queryBuilder = $this->createQueryBuilder();

        $this->prepareCndProduct($queryBuilder, $product);
        $this->prepareCndStatus($queryBuilder, \XLite\Module\XC\Reviews\Model\Review::STATUS_APPROVED, false);

        $queryBuilder->orderBy('r.useForMeta', 'DESC')
            ->addOrderBy('r.rating', 'DESC');

        return $queryBuilder;
    }

    // }}}
}
