<?php
// vim: set ts=4 sw=4 sts=4 et:

/**
 * X-Cart
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the software license agreement
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://www.x-cart.com/license-agreement.html
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to licensing@x-cart.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not modify this file if you wish to upgrade X-Cart to newer versions
 * in the future. If you wish to customize X-Cart for your needs please
 * refer to http://www.x-cart.com/ for more information.
 *
 * @category  X-Cart 5
 * @author    Qualiteam software Ltd <info@x-cart.com>
 * @copyright Copyright (c) 2011-2015 Qualiteam software Ltd <info@x-cart.com>. All rights reserved
 * @license   http://www.x-cart.com/license-agreement.html X-Cart 5 License Agreement
 * @link      http://www.x-cart.com/
 */

namespace XLite\View;

/**
 * Invoice widget
 *
 * 
 */
abstract class InvoiceAbstract extends \XLite\View\AView
{
    /**
     * Widget parameter names
     */

    const PARAM_ORDER = 'order';

    /**
     * Shipping modifier (cache)
     *
     * @var \XLite\Model\Order\Modifier
     */
    protected $shippingModifier;


    /**
     * Get order
     *
     * @return \XLite\Model\Order
     */
    public function getOrder()
    {
        return $this->getParam(self::PARAM_ORDER);
    }

    /**
     * Register CSS files
     *
     * @return array
     */
    public function getCSSFiles()
    {
        $list = parent::getCSSFiles();

        $list[] = 'order/invoice/style.css';

        return $list;
    }


    /**
     * Define widget parameters
     *
     * @return void
     */
    protected function defineWidgetParams()
    {
        parent::defineWidgetParams();

        $this->widgetParams += array(
            self::PARAM_ORDER => new \XLite\Model\WidgetParam\Object(
                'Order', null, false, '\XLite\Model\Order'
            ),
        );
    }

    /**
     * Return default template
     *
     * @return string
     */
    protected function getDefaultTemplate()
    {
        return 'order/invoice/body.tpl';
    }

    /**
     * Check widget visibility
     *
     * @return boolean
     */
    protected function isVisible()
    {
        return parent::isVisible()
            && $this->getOrder();
    }

    /**
     * Get shipping modifier
     *
     * @return \XLite\Model\Order\Modifier
     */
    protected function getShippingModifier()
    {
        if (!isset($this->shippingModifier)) {
            $this->shippingModifier = $this->getOrder()
                ->getModifier(\XLite\Model\Base\Surcharge::TYPE_SHIPPING, 'SHIPPING');
        }

        return $this->shippingModifier;
    }

    /**
     * Get item fescription block columns count 
     * 
     * @return integer
     */
    protected function getItemDescriptionCount()
    {
        return 3;
    }

    /**
     * Get columns span 
     *
     * @return integer
     */
    protected function getColumnsSpan()
    {
        return 4 + count($this->getOrder()->getItemsExcludeSurcharges());
    }

    /**
     * Get payment methods with instructions 
     * 
     * @return array
     */
    protected function getPaymentInstructions()
    {
        $list = array();

        foreach ($this->getOrder()->getVisiblePaymentMethods() as $method) {
            if ($method->getInstruction()) {
                $list[] = $method;
            }
        }

        return $list;
    }

    /**
     * Format invoice price
     *
     * @param float                 $value        Price
     * @param \XLite\Model\Currency $currency     Currency OPTIONAL
     * @param boolean               $strictFormat Flag if the price format is strict (trailing zeroes and so on options) OPTIONAL
     *
     * @return string
     */
    protected function formatInvoicePrice($value, \XLite\Model\Currency $currency = null, $strictFormat = false)
    {
        return static::formatPrice($value, $currency, $strictFormat);
    }

    // {{{ Surcharges

    /**
     * Get surcharge totals
     *
     * @return array
     */
    protected function getSurchargeTotals()
    {
        return $this->getOrder()->getSurchargeTotals();
    }

    /**
     * Get surcharge class name
     *
     * @param string $type      Surcharge type
     * @param array  $surcharge Surcharge
     *
     * @return string
     */
    protected function getSurchargeClassName($type, array $surcharge)
    {
        return 'order-modifier '
            . $type . '-modifier '
            . strtolower($surcharge['code']) . '-code-modifier';
    }

    /**
     * Format surcharge value
     *
     * @param array $surcharge Surcharge
     *
     * @return string
     */
    protected function formatSurcharge(array $surcharge)
    {
        return $this->formatPrice(
            abs($surcharge['cost']),
            $this->getOrder()->getCurrency(),
            \XLite::ADMIN_INTERFACE !== \XLite\Core\Layout::getInstance()->getInterface()
        );
    }

    // }}}

    // {{{ Address

    /**
     * Return specific data for address entry. Helper.
     *
     * @param \XLite\Model\Address $address   Address
     * @param boolean              $showEmpty Show empty fields OPTIONAL
     *
     * @return array
     */
    protected function getAddressSectionData(\XLite\Model\Address $address, $showEmpty = false)
    {
        $data = parent::getAddressSectionData($address, $showEmpty);
        $result = array();

        $name = array(
            'title'     => isset($data['title'])     ? $data['title']     : null,
            'firstname' => isset($data['firstname']) ? $data['firstname'] : null,
            'lastname'  => isset($data['lastname'])  ? $data['lastname']  : null,
        );

        foreach ($data as $serviceName => $field) {
            switch ($serviceName) {
                case 'title':
                case 'firstname':
                case 'lastname':
                    $result += array_filter($name);
                    unset($data['title']);
                    unset($data['firstname']);
                    unset($data['lastname']);
                    break;

                default:
                    $result[$serviceName] = $field;
                    break;
            }
        }

        return $result;
    }

    // }}}

    // {{{ Mail

    /**
     * Returns orders full url in admin area
     *
     * @return string
     */
    protected function getOrderAdminUrl()
    {
        return \XLite\Core\Converter::buildFullURL(
            'order',
            '',
            array('order_number' => $this->getOrder()->getOrderNumber()),
            \XLite::ADMIN_SELF
        );
    }

    /**
     * Returns template for address field
     *
     * @param string $type        Address type
     * @param string $serviceName Field service name
     * @param array  $field       Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFiledTemplate($type, $serviceName, $field)
    {
        $method = 'getAddressField' . \XLite\Core\Converter::convertToCamelCase($serviceName) . 'Template';

        return method_exists($this, $method)
            ? call_user_func(array($this, $method), $type, $field)
            : 'order/invoice/parts/address_field.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldTitleTemplate($type, $field)
    {
        return 'order/invoice/parts/address_field/title.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldFirstnameTemplate($type, $field)
    {
        return 'order/invoice/parts/address_field/firstname.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldLastnameTemplate($type, $field)
    {
        return 'order/invoice/parts/address_field/lastname.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldStreetTemplate($type, $field)
    {
        return 'order/invoice/parts/address_field/street.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldCityTemplate($type, $field)
    {
        return 'order/invoice/parts/address_field/city.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldCountryCodeTemplate($type, $field)
    {
        return 'order/invoice/parts/address_field/country_code.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldStateIdTemplate($type, $field)
    {
        return 'order/invoice/parts/address_field/state_id.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldZipcodeTemplate($type, $field)
    {
        return 'order/invoice/parts/address_field/zipcode.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldPhoneTemplate($type, $field)
    {
        return 'order/invoice/parts/address_field/phone.tpl';
    }

    /**
     * Returns address field template
     *
     * @param string $type  Address type
     * @param array  $field Address field returned by \XLite\View\AView#getAddressSectionData
     *
     * @return string
     */
    protected function getAddressFieldTypeTemplate($type, $field)
    {
        return \XLite\Model\Address::SHIPPING == $type
            ? 'order/invoice/parts/address_field/type.tpl'
            : null;
    }

    // }}
}
